from setuptools import setup, find_packages

setup(
    name="anpm-public",
    version="0.1.0",
    description="A collection of scripts by me, Alia. THIS IS THE PUBLIC VERSION, VERSION  NUMBERS MAY BE INACURRATE.",
    author="Alia Normal",
    author_email="dan.driscoll@aussiebb.com",
    packages=find_packages(),
    install_requires=[],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.12",)
